#include "botCntrlBase.h"


/*********************************************************************************
 */
extern void changeMode(char chMode);
 
boolean BotCntrlBase::bMsgWaiting;
char BotCntrlBase::szInBuf[32];
int BotCntrlBase::iBufUsed;

volatile unsigned long BotCntrlBase::ulRightWheelCnt;
volatile unsigned long BotCntrlBase::ulLeftWheelCnt;

unsigned int BotCntrlBase::ulLastDispUpd;

/*********************************************************************************
 */
void BotCntrlBase::init()
{
    // set up the pins
  pinMode(E1, OUTPUT); 
  pinMode(M1, OUTPUT); 
  pinMode(E2, OUTPUT); 
  pinMode(M2, OUTPUT); 
    
  
  pinMode(leftWheelSensor, INPUT);
  pinMode(rightWheelSensor, INPUT);

  // init the comms/messaging
  bMsgWaiting = false;
  iBufUsed = 0;

  ulLastDispUpd = millis();
}

/*********************************************************************************
 * Check to see if there is a message from the controller (phone) and if so read
 * it in.  Note this overwrites the previous message so make sure you have
 * processed it first.
 */
void BotCntrlBase::checkMsg()
{
  char val;

  while(!bMsgWaiting && Serial.available() > 0)
  {
    val = (char)Serial.read();
    switch(val)
    {
    case -1:
      return;
    case STX:
      iBufUsed = 0;
      break;
    case ETX:
      if(iBufUsed > 0)
        bMsgWaiting = true;
      break;
    case ACK:
    case NAK:
      break;
    default:
      szInBuf[iBufUsed++] = (char)val;
      break;
    }
  }
}

/*********************************************************************************
 * Convert to an int the command received from the controller (phone)
 */
int BotCntrlBase::getCmdVal()
{
  char szVal[CMD_VAL_LEN + 1];
  memcpy(szVal, szInBuf + 2, CMD_VAL_LEN);
  szVal[CMD_VAL_LEN];
  return atoi(szVal);
}

/*********************************************************************************
 */
void BotCntrlBase::setCourseLeft(int iLeftSpeed)
{
  if(iLeftSpeed != iLeftVal)
  {
    iLeftVal = iLeftSpeed;
    bChangeSpeed = true;
  }
}

/*********************************************************************************
 */
int BotCntrlBase::getCourseLeft()
{
  return iLeftVal;
}

/*********************************************************************************
 */
void BotCntrlBase::setCourseRight(int iRightSpeed)
{
  if(iRightSpeed != iRightVal)
  {
    iRightVal = iRightSpeed;
    bChangeSpeed = true;
  }
}

/*********************************************************************************
 */
int BotCntrlBase::getCourseRight()
{
  return iRightVal;
}

/*********************************************************************************
 */
void BotCntrlBase::driveMotors()
{
  if(iLeftVal < 0)
  {
    analogWrite(E1, -iLeftVal);
    digitalWrite(M1, LOW);
  }
  else if(iLeftVal == 0)
  {
    digitalWrite(E1, LOW);
  }
  else
  {
    analogWrite(E1, iLeftVal);
    digitalWrite(M1, HIGH);
  }

  if(iRightVal < 0)
  {
    analogWrite(E2, -iRightVal);
    digitalWrite(M2, LOW);
  }
  else if(iRightVal == 0)
  {
    digitalWrite(E2, LOW);
  }
  else
  {
    analogWrite(E2, iRightVal);
    digitalWrite(M2, HIGH);
  }
}

/*********************************************************************************
 */
void  BotCntrlBase::processMsg(Cmd cmd)
{
  CmdVal toCmdVal;

  switch(cmd.chCmd)
  {
  case CMDS_POWER:
    switch(cmd.chSubCmd)
    {
    case CMD_BAT_LEVEL_REQ:
      toCmdVal.set(CMDS_POWER, CMD_BAT_LEVEL_RESP, analogRead(batSensor));
      break;
  
    case CMD_AMP_SENS_REQ:
      toCmdVal.set(CMDS_POWER, CMD_AMP_SENS_RESP, analogRead(ampSensor));
      break;
      
    default:
      return;
    }
    break;

  case CMDS_OTHER_SENS:
    switch(cmd.chSubCmd)
    {
    case CMD_PROX_SENS_REQ:
      toCmdVal.set(CMDS_OTHER_SENS, CMD_PROX_SENS_RESP, 0);
      if(!digitalRead(leftProxSensor))
      {
        toCmdVal.iVal |= LEFT_PROX_MASK;
      }
      if(!digitalRead(rightProxSensor))
      {
        toCmdVal.iVal |= RIGHT_PROX_MASK;
      }
      break;
      
    default:
      return;
    }
    break;
    
  case CMDS_LITGHT_SENS:

    switch(cmd.chSubCmd)
    {
    case CMD_LIGHT_SENS_LF_REQ:
      toCmdVal.set(CMDS_LITGHT_SENS, CMD_LIGHT_SENS_LF_RESP, analogRead(leftFrontLightSensor));
      break;
  
    case CMD_LIGHT_SENS_RF_REQ:
      toCmdVal.set(CMDS_LITGHT_SENS, CMD_LIGHT_SENS_RF_RESP, analogRead(leftFrontLightSensor));
      break;
  
    case CMD_LIGHT_SENS_LB_REQ:
      toCmdVal.set(CMDS_LITGHT_SENS, CMD_LIGHT_SENS_LB_RESP, analogRead(leftFrontLightSensor));
      break;
  
    case CMD_LIGHT_SENS_RB_REQ:
      toCmdVal.set(CMDS_LITGHT_SENS, CMD_LIGHT_SENS_RB_RESP, analogRead(leftFrontLightSensor));
      break;

    default:
      return;
    }
    break;

  case CMDS_MODE:
    switch(cmd.chSubCmd)
    {  
    case CMD_REMOTE_CONTROL:
    case CMD_SET_PATH:
    case CMD_SEEK_LIGHT:
      changeMode(cmd.chSubCmd);
      break;
    }
    return;
    
  default:
    return;
  }
  toCmdVal.serialSend();
}

/*********************************************************************************
 */
void  BotCntrlBase::processMsg(CmdVal cmdVal)
{
}

/*********************************************************************************
 * Process the message as recieved from the controller (phone)
 */
void BotCntrlBase::processMsg()
{
  CmdVal toSend;
  toSend.chCmd = szInBuf[0];
  toSend.chSubCmd = szInBuf[1];

  // extract the value if supplied
  if(iBufUsed == (2 + CMD_VAL_LEN))
  {
    toSend.iVal = getCmdVal();
    
    processMsg(toSend);
  }
  else
  {
    processMsg((Cmd)toSend);
  }
  
  // now the final common stuff
  bMsgWaiting = false;
}

/*********************************************************************************
 */
void BotCntrlBase::sendSensToDisplay()
{
  int iBat = analogRead(batSensor);
  float fAmp = analogRead(ampSensor) - 512;
  fAmp *= (100.0 / 1024.0) / (.185 * 2);
  int iLF = analogRead(leftFrontLightSensor);
  int iRF = analogRead(rightFrontLightSensor);
  int iLB = analogRead(leftBackLightSensor);
  int iRB = analogRead(rightBackLightSensor);
  int iProx = 0;
  if(!digitalRead(leftProxSensor))
    iProx += LEFT_PROX_MASK;
  if(!digitalRead(rightProxSensor))
    iProx += RIGHT_PROX_MASK;

  CmdVal val;
  val.set(CMDS_POWER, CMD_BAT_LEVEL_RESP, iBat);
  val.i2cSend(I2C_DISPLAY);

  val.set(CMDS_POWER, CMD_AMP_SENS_RESP, (int)(fAmp * 10));
  val.i2cSend(I2C_DISPLAY);

  val.set(CMDS_LITGHT_SENS, CMD_LIGHT_SENS_LF_RESP, iLF);
  val.i2cSend(I2C_DISPLAY);

  val.set(CMDS_LITGHT_SENS, CMD_LIGHT_SENS_RF_RESP, iRF);
  val.i2cSend(I2C_DISPLAY);

  val.set(CMDS_LITGHT_SENS, CMD_LIGHT_SENS_LB_RESP, iLB);
  val.i2cSend(I2C_DISPLAY);

  val.set(CMDS_LITGHT_SENS, CMD_LIGHT_SENS_RB_RESP, iRB);
  val.i2cSend(I2C_DISPLAY);

  val.set(CMDS_OTHER_SENS, CMD_PROX_SENS_RESP, iProx);
  val.i2cSend(I2C_DISPLAY);
}

/*********************************************************************************
 */
void BotCntrlBase::displayLight(Cmd cmd)
{
  cmd.i2cSend(I2C_DISPLAY);
}

/*********************************************************************************
 */
void BotCntrlBase::doLoop()
{
  int iTmp;
  unsigned long ulNow = millis();

  // get the details from the controller
  if(!bMsgWaiting)
  {
    checkMsg();
  }
  if(bMsgWaiting)
  {
    processMsg();
  }

  if(ulNow < ulLastDispUpd)
  {
    ulLastDispUpd = ulNow;
  }
  if(ulNow - ulLastDispUpd > 500)
  {
    sendSensToDisplay();
    ulLastDispUpd = ulNow;
  }
  readSensors();

  if(bChangeSpeed)
  {
    driveMotors();
  }
}

/*********************************************************************************
 */
void BotCntrlBase::restarting()
{
    iLeftVal = iRightVal = 0;
    bChangeSpeed = true;
}

